/*
 * fc.c
 *
 *  Created on: Nov 6, 2010
 *      Author: mike
 */

#include "fc.h"

// Body fixed rates
#define						p				BodyRate_i32_m1024__deg_d_s[0]
#define						q				BodyRate_i32_m1024__deg_d_s[1]
#define						r				BodyRate_i32_m1024__deg_d_s[2]
// Euler rates
#define						phi_dot			EulerRate_i32_m1024__deg_d_s[0]
#define						theta_dot		EulerRate_i32_m1024__deg_d_s[1]
#define						psi_dot			EulerRate_i32_m1024__deg_d_s[2]
// Euler Angles (filter)
#define						phi				EulerAngle_Filtered_i32_m1024__deg[0]
#define						theta			EulerAngle_Filtered_i32_m1024__deg[1]
#define						psi				EulerAngle_Filtered_i32_m1024__deg[2]
// Euler Angles (accelerometer & magnetometer)
#define						phi_acc			EulerAngle_Accel_i32_m1024__deg[0]
#define						theta_acc		EulerAngle_Accel_i32_m1024__deg[1]
#define						psi_mag			EulerAngle_Accel_i32_m1024__deg[2]
// Trigonometric functions
#define						sini(_param)	( sin_i32_m1024(_param) )
#define						cosi(_param)	( cos_i32_m1024(_param) )
#define						tani(_param)	( tan_i32_m1024(_param) )

// Variables
int16_t						AccelRaw_i16_m1024__1g[3];
int32_t						AccelVectorLength;
uint8_t						UseAccelForFilterFlag = 1;
int16_t						GyroRaw_i16_m14d375__deg_d_s[3];

int32_t						FilterWeight = 41;										// Complementary filter weighting value

int32_t						BodyRate_i32_m1024__deg_d_s[3]			= {0, 0, 0};	// Body fixed rates, p (index 0), q, r
int32_t						EulerRate_i32_m1024__deg_d_s[3]			= {0, 0, 0};	// Euler rates, phi_dot, theta_dot, psi_dot
int32_t						EulerAngle_Accel_i32_m1024__deg[3]		= {0, 0, 0};	// Euler angles (from accelerometer) phi, theta, (psi)
int32_t						EulerAngle_Filtered_i32_m1024__deg[3]	= {0, 0, 0};	// Euler angles (filtered) phi, theta, (psi)

int16_t						Debug_BodyRate_i16_m8__deg_d_s[3];
int16_t						Debug_EulerRate_i16_m8__deg_d_s[3];
int16_t						Debug_EulerAngle_Accel_i16_m8__deg[3];
int16_t						Debug_EulerAngle_Filter_i16_m8__deg[3];


/*****************************************************************************************************************************************/
void FlightControl ()
{
	uint8_t	i;

	// Check for new LIS values
	if (UpdateLISSensorValues ())
	{
		if ( (AccelVectorLength > (629146)) && (AccelVectorLength < 1468006))
		{
			FilterWeight = 41;
		}
		else
		{
			FilterWeight = 0;
		}
	}

	// Check for new ITG values
	if (UpdateITGSensorValues ())
	{
		// Convert and scale body fixed rates p, q, r
		// GyroRaw_i16_m14d375__deg_d_s range: +-2000 * 14.375
		p			=	(int32_t)(((int32_t) GyroRaw_i16_m14d375__deg_d_s[2] * 8192L) / 115L);						// p [deg/s]
		q			=	(int32_t)(((int32_t)-GyroRaw_i16_m14d375__deg_d_s[0] * 8192L) / 115L);						// q [deg/s]
		r			=	(int32_t)(((int32_t)-GyroRaw_i16_m14d375__deg_d_s[1] * 8192L) / 115L);						// r [deg/s]

		// Calculate Euler rates from body rates (body fixed -> earth fixed)
		// Euler_vector_dot = Jacobi_Matrix * BodyRate_vector
		// Calculate Phi dot
		phi_dot			=	  p +
							((q * ((sini(phi) * tani(theta)) / 1024L)) / 1024L) +
							((r * ((cosi(phi) * tani(theta)) / 1024L)) / 1024L);
		// Calculate Theta dot
		theta_dot		=	((q * cosi(phi)) / 1024L) -
							((r * sini(phi)) / 1024L);
		// Calculate Psi dot
		psi_dot			=	((q * sini(phi)) / cosi(theta) ) +
							((r * cosi(phi)) / cosi(theta) );

		// Calculate Euler angles from acceleration values
		phi_acc			=	atan2_i32_m1024 (-AccelRaw_i16_m1024__1g[1], AccelRaw_i16_m1024__1g[0]);
		theta_acc		=	atan2_i32_m1024 ( AccelRaw_i16_m1024__1g[2], AccelRaw_i16_m1024__1g[0]);

		EulerAngle_Accel_i32_m1024__deg[2]		=   0;																					// Value from magnetometer

		int32_t RollIntegration_i32_m1024__deg	=	phi_dot		/ CONTROL_FREQUENCY_HZ;
		int32_t PitchIntegration_i32_m1024__deg	=	theta_dot	/ CONTROL_FREQUENCY_HZ;
		int32_t	YawIntegration_i32_m1024__deg	=	psi_dot		/ CONTROL_FREQUENCY_HZ;

		// Complementary filter
		phi		=	((4096L - FilterWeight)	* (phi + RollIntegration_i32_m1024__deg) +
					 (FilterWeight) 		* (phi_acc)) / 4096L;
		theta	=	((4096L - FilterWeight)	* (theta + PitchIntegration_i32_m1024__deg) +
					 (FilterWeight) 		* (theta_acc)) / 4096L;
		psi		=	((4096L - 2048L)		* (psi + YawIntegration_i32_m1024__deg) +
					 (2048) 				* (psi)) / 4096L;

		// Debug & Logging values
		for (i = 0; i < 3; i++)
		{
			Debug_BodyRate_i16_m8__deg_d_s[i]		= (int16_t) (BodyRate_i32_m1024__deg_d_s[i]			/ 128L);
			Debug_EulerRate_i16_m8__deg_d_s[i]		= (int16_t)	(EulerRate_i32_m1024__deg_d_s[i]		/ 128L);
			Debug_EulerAngle_Accel_i16_m8__deg[i]	= (int16_t) (EulerAngle_Accel_i32_m1024__deg[i]		/ 128L);
			Debug_EulerAngle_Filter_i16_m8__deg[i]	= (int16_t) (EulerAngle_Filtered_i32_m1024__deg[i]	/ 128L);
		}

		SensorDebug ();
	}
}

/*****************************************************************************************************************************************/
void SensorDebug ()
{
	static uint8_t	count = 0;

	uint8_t	i, ascii = 1;

	if (count++ == 9)
	{
		count = 0;
		LED_BLUE_TOGGLE;
		PORTD.OUTTGL = (1 << PIN0);
	}

	if (ascii == 0)
	{
		Debug_ch (0xA6);
		Debug_ch (0x8C);

		for (i = 0; i < 6; i++)
			Debug_ch (((char *)Debug_BodyRate_i16_m8__deg_d_s)[i]);

		for (i = 0; i < 6; i++)
			Debug_ch (((char *)Debug_EulerRate_i16_m8__deg_d_s)[i]);

		for (i = 0; i < 6; i++)
			Debug_ch (((char *)AccelRaw_i16_m1024__1g)[i]);

		for (i = 0; i < 6; i++)
			Debug_ch (((char *)Debug_EulerAngle_Accel_i16_m8__deg)[i]);

		for (i = 0; i < 6; i++)
			Debug_ch (((char *)Debug_EulerAngle_Filter_i16_m8__deg)[i]);
	}
	else if (ascii == 1)
	{
		Debug_Str ("p,q,r: ");
		for (i = 0; i < 3; i++)
			Debug_i16 (Debug_BodyRate_i16_m8__deg_d_s[i] / 8);

		Debug_Str ("   ax,ay,az: ");
		for (i = 0; i < 3; i++)
			Debug_i16 (AccelRaw_i16_m1024__1g[i]);

		Debug_Str ("   AccelAngles: ");
		for (i = 0; i < 3; i++)
			Debug_i16 (Debug_EulerAngle_Accel_i16_m8__deg[i] / 8);

		Debug_Str ("   FilterAngles: ");
		for (i = 0; i < 3; i++)
			Debug_i16 ((int16_t)(Debug_EulerAngle_Filter_i16_m8__deg[i] / 8));

		Debug_Str ("\n");
	}

}

/*****************************************************************************************************************************************/
uint8_t UpdateLISSensorValues ()
{
	if (Get_LIS_LastActionStatus() == LIS_ACTION_GETXYZ)
	{
		Get_LISRawValues (AccelRaw_i16_m1024__1g);

		// Calculate resulting acceleration vector length²
		AccelVectorLength = ((int32_t)AccelRaw_i16_m1024__1g[0] * (int32_t)AccelRaw_i16_m1024__1g[0]) +
				((int32_t)AccelRaw_i16_m1024__1g[1] * (int32_t)AccelRaw_i16_m1024__1g[1]) +
				((int32_t)AccelRaw_i16_m1024__1g[2] * (int32_t)AccelRaw_i16_m1024__1g[2]);

		return 1;
	}


	return 0;
}

/*****************************************************************************************************************************************/
uint8_t UpdateITGSensorValues ()
{
	if (GetLastITGActionStatus() == ITG_ACTION_GETTEMPXYZ)
	{
		Get_ITGRawValues (GyroRaw_i16_m14d375__deg_d_s);
		return 1;
	}

	return 0;
}



















