/*
 * itg3200.c
 *
 *  Created on: 15.10.2010
 *      Author: mkriegel
 */

#include "itg3200.h"

volatile uint8_t			ITGActionStatus = ITG_ACTION_IDLE;				// Current action status (see E_ITG_ACTION_STATES in header)
volatile uint8_t			ITGByteCount = 0;							// internal byte counter
volatile uint8_t			LastITGActionStatus = ITG_ACTION_FAILURE;	// Result is 0 = Failure or successful action state defined in E_ITG_ACTION_STATES (see header)
volatile uint8_t		*	DataBuffer;									// Pointer to buffer where data shall be written to

volatile int16_t			Temperature_m10;							// Temperature value * 10
volatile uint8_t			GyroByteData[8];							// Raw gyro byte values 0,1=X - 2,3=Y - 4,5=Z - 6,7=Temp
volatile int16_t			GyroRawData[3];								// Raw gyro values 0=X 1=Y 2=Z

#if (ITG_QUERY_WITH_TIMER == 1)
/*****************************************************************************************************************************************/
ISR (ITG_UPDATE_TIMER_INT)
{
	ITG_Debug_Str ("ITG TOVL\n");

	if (ITGActionStatus == ITG_ACTION_IDLE)
	{
		StartITGAction(ITG_ACTION_GETTEMPXYZ, GyroByteData, FUNC_NON_BLOCKING);
	}
	else
	{
		TWI_STOP (&ITG_TWI_PORT);																// TWI stop condition
		ITGActionStatus					= ITG_ACTION_IDLE;											// Reset ITG status state machine
		ITGByteCount				= 0;														// Reset ITGByteCount
	}
}
#else
/*****************************************************************************************************************************************/
ISR (ITG_INT_ISR_VECTOR)
{
	if (ITGActionStatus == ITG_ACTION_IDLE)
		StartITGAction(ITG_ACTION_GETTEMPXYZ, FUNC_NON_BLOCKING);
}
#endif


/*****************************************************************************************************************************************/
ISR (ITG_TWI_INT)
{
	ITG_Debug_Str	("ITG-INT bc: ");															// Debug output
	ITG_Debug_u8	(ITGByteCount);																// Debug output
	ITG_Debug_Str	(" ");																		// Debug output

	uint8_t	status = ITG_TWI_PORT.MASTER.STATUS;												// Get status from TWI Master

	if (status & TWI_MASTER_ARBLOST_bm)															// Check if TWI ARB was lost
		ITG_Handle_TWI_ARB_Lost ();																// Recover from ARB lost

	else if (status & TWI_MASTER_BUSERR_bm)														// Check if TWI bus error happened
		ITG_Handle_TWI_BUS_Error ();															// Recover from BUS error

	else if (status & TWI_MASTER_WIF_bm)														// Check if WIF (write interrupt flag) is set
		ITG_Handle_TWI_WIF ();																	// Handle TWI WIF

	else if (status & TWI_MASTER_RIF_bm)														// Check if RIF (read interrupt flag) is set
		ITG_Handle_TWI_RIF ();																	// Handle TWI RIF

	else																						// Some other unexpected condition happened
		ITG_Handle_TWI_Unexpected ();															// Handle it!
}

/*************************************************************************************************************/
void ITG_Handle_TWI_RIF ()
{
	ITG_Debug_Str ("RIF: ");																	// Debug output
	uint8_t	data = ITG_TWI_PORT.MASTER.DATA;													// Read byte from TWI data register
	ITG_Debug_u8	(data);																		// Debug output
	ITG_Debug_Str	(" ");																		// Debug output

	switch (ITGActionStatus)																	// Switch "action" state machine status
	{
		case ITG_ACTION_GETTEMPXYZ:																// Read temperature and XYZ gyro values
			if (ITGByteCount < ITG_TXYZ_LEN)													// One of eight 16er complement bytes
			{
				GyroByteData[ITGByteCount] = data;												// Write received data byte to buffer
				TWI_ACK (&ITG_TWI_PORT);														// Issue ACK and start a new byte read
				ITGByteCount++;																	// Increment ITGByteCount
				ITG_Debug_Str ("\n");															// Debug output
			}
			else																				// Receive last byte and stop TWI reading
			{
				GyroByteData[ITGByteCount] = data;												// Write received data byte to buffer
				TWI_NACK_STOP (&ITG_TWI_PORT);													// Issue NACK and send stop condition
				LastITGActionStatus		= ITG_ACTION_GETTEMPXYZ;								// Set last action result
				ITGActionStatus			= ITG_ACTION_IDLE;										// Reset ITG status state machine
				ITGByteCount			= 0;													// Reset ITGByteCount
				GyroRawData[0] 			= (int16_t)((GyroByteData[2] << 8) + GyroByteData[3]);	// Calc p * 14,375 [deg / s]
				GyroRawData[1] 			= (int16_t)((GyroByteData[4] << 8) + GyroByteData[5]);	// Calc q * 14,375 [deg / s]
				GyroRawData[2] 			= (int16_t)((GyroByteData[6] << 8) + GyroByteData[7]);	// Calc r * 14,375 [deg / s]
				Temperature_m10			= 350 + (((GyroByteData[0] << 8) +
						GyroByteData[1]) + 13200) / 28;											// Calc temperature * 10

				ITG_Debug_Str ("DATA OK\n");													// Debug output
			}
			break;
	}
}

/*************************************************************************************************************/
inline void ITG_Handle_TWI_Unexpected ()
{
	TWI_STOP(&ITG_TWI_PORT);																	// Send TWI stop condition
	ITG_ERROR_Debug_Str ("ITG unexpected condition\n");											// Debug output
	ITGActionStatus			= ITG_ACTION_FAILURE;												// Reset ITG status state machine
	ITGByteCount			= 0;																// Reset ITGByteCount
}

/*************************************************************************************************************/
inline void ITG_Handle_TWI_WIF ()
{
	ITG_Debug_Str ("WIF ");																		// Debug output

	if (ITG_TWI_PORT.MASTER.STATUS & TWI_MASTER_RXACK_bm)										// Check if NACK received
		ITG_Handle_TWI_WIF_NACK ();																// Handle ITG WIF NACK
	else																						// ACK received
		ITG_Handle_TWI_WIF_ACK	();																// Handle ITG WIF ACK
}

/*************************************************************************************************************/
inline void ITG_Handle_TWI_WIF_NACK ()
{
	TWI_STOP (&ITG_TWI_PORT);																	// TWI stop condition
	ITG_ERROR_Debug_Str ("ITG NACK\n");															// Debug output
	LastITGActionStatus		= ITG_ACTION_FAILURE;												// Set last action result
	ITGActionStatus			= ITG_ACTION_IDLE;													// Reset ITG status state machine
}

/*************************************************************************************************************/
inline void ITG_Handle_TWI_WIF_ACK ()
{
	ITG_Debug_Str ("ACK ");																		// Debug output

	//ITG_Debug_u8(ITGStatus);

	switch (ITGActionStatus)																	// Switch "action" state machine status
	{
		// *****************************************************************************
		case ITG_ACTION_IDLE:
			TWI_STOP (&ITG_TWI_PORT);															// TWI stop condition
			ITGActionStatus			= ITG_ACTION_IDLE;											// Reset ITG status state machine
			ITGByteCount			= 0;														// Reset ITGByteCount
			ITG_Debug_Str ("ITG_CHECK_COMM SUCCESSFUL - TWI stopped\n");						// Debug output
			break;

			// *****************************************************************************
		case ITG_ACTION_CHECK_COMM:																// COMM CHECK +++++++++++++++++++++++++++++++++++++++++++++
			TWI_STOP (&ITG_TWI_PORT);															// TWI stop condition
			LastITGActionStatus		= ITG_ACTION_CHECK_COMM;									// Set last action result
			ITGActionStatus			= ITG_ACTION_IDLE;												// Reset ITG status state machine
			ITGByteCount			= 0;														// Reset ITGByteCount
			ITG_Debug_Str ("ITG_CHECK_COMM SUCCESSFUL - TWI stopped\n");						// Debug output
			break;

			// *****************************************************************************
		case ITG_ACTION_CONFIGURE:																// Set sample rate
			// ITG Sensor reset
			if (ITGByteCount == 0)																// Reset ITG sensor
			{
				TWI_WRITE(&ITG_TWI_PORT, ITG_PWR_MGM);											// Write power management register address
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("ITG_PWR_MGM\n");
			}
			else if (ITGByteCount == 1)															// Set device reset flag in power management register address
			{
				TWI_WRITE(&ITG_TWI_PORT, 0x80);													// Set device reset flag
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("0x80\n");
			}
			// Repeated start ---------------------------------------------------------
			// ITG Sensor set sample rate divider
			else if (ITGByteCount == 2)															// Send repeated start condition
			{
				TWI_START (&ITG_TWI_PORT, ITG + I2C_WRITE);										// Send repeated start condition
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("RepStart\n");
			}
			else if (ITGByteCount == 3)															// Send sample rate register address
			{
				TWI_WRITE(&ITG_TWI_PORT, ITG_SMPLRT_DIV);										// Write sample rate divider register address
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("ITG_SMPLRT_DIV\n");
			}
			else if (ITGByteCount == 4)															// Send sample rate config value
			{
				TWI_WRITE(&ITG_TWI_PORT, ITG_SAMPLE_RATE_DIVIDER);								// 0x13 = 19d; Desired sample rate = 400Hz (see manual)
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("ITG_SAMPLE_RATE_DIVIDER\n");
			}
			// Repeated start ---------------------------------------------------------
			// ITG Sensor set digital low pass filter (8khz internal frequency, 256Hz low pass filter bandwidth)
			else if (ITGByteCount == 5)															// Send repeated start condition
			{
				TWI_START (&ITG_TWI_PORT, ITG + I2C_WRITE);										// Send repeated start condition
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("RepStart\n");
			}
			else if (ITGByteCount == 6)															// Send DLPF register address
			{
				TWI_WRITE (&ITG_TWI_PORT, ITG_DLPF_FS);											// Write DLPF register address
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("ITG_DLPF_FS\n");
			}
			else if (ITGByteCount == 7)															// Send DLPF config value
			{
				TWI_WRITE (&ITG_TWI_PORT, 0x18);												// Write DLPF config value (see manual)
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("0x18\n");
			}
			// Repeated start ---------------------------------------------------------
			else if (ITGByteCount == 8)															// Send repeated start condition
			{
				TWI_START (&ITG_TWI_PORT, ITG + I2C_WRITE);										// Send repeated start condition
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("RepStart\n");
			}
			else if (ITGByteCount == 9)															// Send interrupt configuration register address
			{
				TWI_WRITE (&ITG_TWI_PORT, ITG_INT_CFG);											// Write interrupt configuration register address
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("ITG_INT_CFG\n");
			}
			else if (ITGByteCount == 10)														// Send interrupt configuration register address
			{
				TWI_WRITE (&ITG_TWI_PORT, 0x05);												// Write ITG interrupt config (enable ITG_RDY_EN & RAW_RDY_EN)
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("0x05\n");
			}
			// Repeated start ---------------------------------------------------------
			else if (ITGByteCount == 11)														// Send repeated start condition
			{
				TWI_START (&ITG_TWI_PORT, ITG + I2C_WRITE);										// Send repeated start condition
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("RepStart\n");
			}
			else if (ITGByteCount == 12)														// Send power management register address
			{
				TWI_WRITE (&ITG_TWI_PORT, ITG_PWR_MGM);											// Write power management register address
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("ITG_PWR_MGM\n");
			}
			else if (ITGByteCount == 13)														// Set clock reference to PLL with Gyro X reference
			{
				TWI_WRITE (&ITG_TWI_PORT, 0x01);												// Write clock reference to PLL with Gyro X reference (see manual)
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("0x01\n");
			}
			else
			{
				TWI_STOP (&ITG_TWI_PORT);														// TWI stop condition
				LastITGActionStatus		= ITG_ACTION_CONFIGURE;									// Set last action result
				ITGActionStatus			= ITG_ACTION_IDLE;										// Reset ITG status state machine
				ITGByteCount			= 0;													// Reset ITGByteCount
				ITG_Debug_Str ("ITG configured.\n");											// Debug output
			}
			break;

			// *****************************************************************************
		case ITG_ACTION_GETTEMPXYZ:																// Get temperature and XYZ gyro value
			if (ITGByteCount == 0)																// Send register address from which to read from
			{
				TWI_WRITE (&ITG_TWI_PORT, ITG_TEMP_OUT_H);										// Write register address of temperature msb
				ITGByteCount++;																	// Increase ITGByteCount
				ITG_Debug_Str ("W-Reg\n");
			}
			else if (ITGByteCount == 1)															// Switch transmission to reading
			{
				TWI_START (&ITG_TWI_PORT, ITG + I2C_READ);										// Send ITG read address
				ITGByteCount = 0;																// Reset ITGByteCount for reading
				ITG_Debug_Str ("RS-Read\n");
			}
			break;
	}
}

/*************************************************************************************************************/
inline void	ITG_Handle_TWI_BUS_Error ()
{
	TWI_STOP (&ITG_TWI_PORT);																	// TWI stop condition
	ITG_ERROR_Debug_Str ("ITG BUSERR\n");														// Debug output
	ITG_TWI_PORT.MASTER.STATUS |= TWI_MASTER_BUSERR_bm;											// Reset BUS error flag
	ITGActionStatus				= ITG_ACTION_IDLE;												// Reset ITG status state machine
	ITGByteCount				= 0;															// Reset ITGByteCount
}

/*************************************************************************************************************/
inline void ITG_Handle_TWI_ARB_Lost ()
{
	TWI_STOP (&ITG_TWI_PORT);																	// TWI stop condition
	ITG_ERROR_Debug_Str ("ITG ARBLOST\n");														// Debug output
	ITG_TWI_PORT.MASTER.STATUS |= TWI_MASTER_ARBLOST_bm;										// Reset ARB lost flag
	ITGActionStatus				= ITG_ACTION_IDLE;												// Reset ITG status state machine
	ITGByteCount				= 0;															// Reset ITGByteCount

	Recover_ITG_TWI_from_ARBLost ();															// Recover from ARB lost condition
}

/*************************************************************************************************************/
inline void Recover_ITG_TWI_from_ARBLost()
{
	Disable_ITG_TWI ();

	ITG_PORT.DIRSET = (1 << PIN1);
	ITG_PORT.DIRCLR = (1 << PIN0);

	while (!(ITG_PORT.IN & (1 << PIN0)))
	{
		Debug_Str ("ITG RECOVER");
		ITG_PORT.OUTTGL = (1 << PIN1);
		_delay_us(10);
	}

	Init_ITG_TWI();
	ITG_ERROR_Debug_Str ("Recover_ITG_TWI_from_ARBLost\n");
}

/*************************************************************************************************************/
void Disable_ITG_TWI ()
{
	ITG_TWI_PORT.MASTER.BAUD	= 0;
	ITG_TWI_PORT.MASTER.CTRLA	= 0;
	ITG_TWI_PORT.MASTER.STATUS	= TWI_MASTER_BUSSTATE_IDLE_gc;
}

/*************************************************************************************************************/
void Init_ITG_TWI ()
{
	// Init ITG TWI port and baudrate
	ITG_TWI_PORT.MASTER.BAUD	= TWI_BAUD(SYS_CLK, ITG_TWI_FREQUENCY_HZ);
	ITG_TWI_PORT.MASTER.CTRLA	= TWI_MASTER_INTLVL_HI_gc |
			TWI_MASTER_ENABLE_bm |
			TWI_MASTER_RIEN_bm |
			TWI_MASTER_WIEN_bm;
	ITG_TWI_PORT.MASTER.STATUS	= TWI_MASTER_BUSSTATE_IDLE_gc;
}

/*************************************************************************************************************/
uint8_t Init_ITG_Sensor ()
{
	Init_ITG_TWI ();

	_delay_ms (50);

	Debug_Str ("Checking ITG communications:\n");
	StartITGAction(ITG_ACTION_CHECK_COMM, FUNC_BLOCKING);
	if (GetLastITGActionStatus() == ITG_ACTION_CHECK_COMM)
		Debug_Str ("  -ITG-3200 3 axes gyro found ...\n");
	else
	{
		Debug_Str ("  - FAILURE\n");
		return 0;
	}

	_delay_ms (50);

	Debug_Str ("Configure ITG sensor:\n");
	StartITGAction(ITG_ACTION_CONFIGURE, FUNC_BLOCKING);
	if (GetLastITGActionStatus() == ITG_ACTION_CONFIGURE)
	{
		Debug_Str		("  -ITG-3200 3 axes gyro configured ...\n");
		Debug_Str		("  -Sample rate [Hz]: ");
		Debug_i16_left	(ITG_SAMPLE_RATE_HZ);
		Debug_Str		("\n");
		Debug_Str		("  -Sample rate divider: ");
		Debug_u8		(ITG_SAMPLE_RATE_DIVIDER);
		Debug_Str		("\n\n");
	}
	else
	{
		Debug_Str (" - FAILURE\n");
		return 0;
	}

	_delay_ms (50);

	return 1;
}

/*************************************************************************************************************/
inline int16_t Get_ITG_Temperature ()
{
	return Temperature_m10;
}

/*************************************************************************************************************/
inline void Get_ITGRawValues (int16_t * DataBuffer)
{
	uint8_t i;

	cli ();

	for (i = 0; i < 3; i++)
		DataBuffer[i] = GyroRawData[i];

	sei ();

	LastITGActionStatus = 0;
}


/*************************************************************************************************************/
inline void StartITGDataQuery ()
{
#if (ITG_QUERY_WITH_TIMER == 1)
	// Init ITG timer
	TC_SetPeriod 		(&ITG_UPDATE_TIMER, SYS_CLK / ITG_UPDATE_FREQUENCY_HZ / ITG_TIMER_PRESCALER_VALUE);
	TC_SetPrescaler		(&ITG_UPDATE_TIMER, ITG_TIMER_PRESCALER);
	TC_SetOverflowInt	(&ITG_UPDATE_TIMER, TC_OVFINTLVL_HI_gc);
#else
	ITG_INT_PORT.DIRCLR 	= (1 << ITG_INT_PIN);									// Make ITG interrupt pin an input
	PORTCFG.MPCMASK			= (1 << ITG_INT_PIN);									// Mask ITG input pin
	ITG_INT_PORT.PIN0CTRL 	= PORT_OPC_PULLDOWN_gc;									// Enable pulldown resistor at ITG input pin
	ITG_INT_PORT.INTCTRL 	|= PORT_INT0LVL_MED_gc;									// Enable medium level interrupt
	ITG_INT_PORT.INT0MASK 	= (1 << ITG_INT_PIN);									// Mask ITG input pin for interrupt 0
	ITG_INT_PORT.PIN2CTRL 	= PORT_ISC_RISING_gc;									// Enable interrupt on rising edge
#endif
}

/*************************************************************************************************************/
inline uint8_t GetLastITGActionStatus ()
{
	return LastITGActionStatus;
}

/*************************************************************************************************************/
inline uint8_t	GetITGActionStatus ()
{
	return ITGActionStatus;
}

/*************************************************************************************************************/
uint8_t	StartITGAction (uint8_t ActionType, uint8_t Blocking)
{
	ITG_Debug_Str	("\nITG-ACT: ");																	// Debug output
	ITG_Debug_u8	(ActionType);																		// Debug output
	ITG_Debug_Str	("\n");																				// Debug output

	if (ActionType > ITG_ACTION_NUMBER_OF_STATES)														// Check if action is valid
	{
		ITG_Debug_Str ("ITG action is unknown\n");														// Debug output
		ITGActionStatus = ITG_ACTION_IDLE;																// Reset current action
		return ITGActionStatus;																			// Return from function
	}
	else if (ITGActionStatus != ITG_ACTION_IDLE)														// Check if no action is pending
	{
		ITG_Debug_Str ("ITG ACTION PENDING\n");															// Debug output
		return ITGActionStatus;																			// Return from function
	}
	else																								// Start action
	{
		ITGActionStatus		= ActionType;																// Set current action type
		ITGByteCount		= 0;																		// Reset byte count
	}

	ITG_Debug_Str ("ITG-BUS: ");																		// Debug output
	switch (ITG_TWI_PORT.MASTER.STATUS & TWI_MASTER_BUSSTATE_gm)										// Debug current bus state
	{
		case TWI_MASTER_BUSSTATE_UNKNOWN_gc:
			ITG_Debug_Str ("UNKNOWN\n");
			break;

		case TWI_MASTER_BUSSTATE_IDLE_gc:
			ITG_Debug_Str ("IDLE\n");
			break;

		case TWI_MASTER_BUSSTATE_OWNER_gc:
			ITG_Debug_Str ("OWNER\n");
			break;

		case TWI_MASTER_BUSSTATE_BUSY_gc:
			ITG_Debug_Str ("BUSY\n");
			break;
	}

	// Start condition + ITG address
	ITG_Debug_Str	("ADDR: ");																			// Debug output
	ITG_Debug_u8	(ITG);																				// Debug output
	ITG_Debug_Str	("\n");																				// Debug output
	TWI_START (&ITG_TWI_PORT, ITG + I2C_WRITE);															// Start TWI

	if (Blocking)																						// Check if function shall block
	{
		while (ITGActionStatus != ITG_ACTION_IDLE)
		{
			Debug_Str ("BLOCK");
			asm volatile ("nop");
		}
		ITG_Debug_Str ("ITG action done ...\n\n");														// Debug output
	}

	return ITGActionStatus;
}




