/*
 * itg3200.h
 *
 *  Created on: 15.10.2010
 *      Author: mkriegel
 */

#ifndef ITG3200_H_
#define ITG3200_H_

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "../tricopter_config.h"
#include "../m_twi.h"
#include "../m_timer.h"
#include "../debug.h"

#define		ITG_INTERNAL_SAMPLE_RATE_HZ		8000UL
#define		ITG_GYRO_SCALE_M1024			(1024L * 1024L / 14720L)

#ifndef		ITG_SAMPLE_RATE_HZ
#error		"ITG SAMPLE RATE NOT FOUND"
#else
#define		ITG_SAMPLE_RATE_DIVIDER			( (ITG_INTERNAL_SAMPLE_RATE_HZ / ITG_SAMPLE_RATE_HZ) - 1 )
#endif

#if (ITG_DEBUGGING == 1)
#define		ITG_Debug_Str(_str)					( Debug_Str(_str) )
#define		ITG_Debug_u8(_u8)					( Debug_u8(_u8) )
#else
#define		ITG_Debug_Str(_str)
#define		ITG_Debug_u8(_u8)
#endif

#if (ITG_ERROR_DEBUGGING == 1)
#define 	ITG_ERROR_Debug_Str(_str)			( Debug_Str(_str) )
#else
#define		ITG_ERROR_Debug_Str(_str)
#endif

// Timer settings calculation -- do NOT change
#ifndef SYS_CLK
	#error "SYS_CLK: System clock frequency not defined!!!"
#endif

#if (ITG_QUERY_WITH_TIMER == 1)
	#if ((SYS_CLK / ITG_UPDATE_FREQUENCY_HZ) < 65535)
		#define		ITG_TIMER_PRESCALER_VALUE		1UL
		#define		ITG_TIMER_PRESCALER				TC_CLKSEL_DIV1_gc
	#elif ((SYS_CLK / ITG_UPDATE_FREQUENCY_HZ / 2UL) < 65535)
		#define		ITG_TIMER_PRESCALER_VALUE		2UL
		#define		ITG_TIMER_PRESCALER				TC_CLKSEL_DIV2_gc
	#elif ((SYS_CLK / ITG_UPDATE_FREQUENCY_HZ / 4UL) < 65535)
		#define		ITG_TIMER_PRESCALER_VALUE		4UL
		#define		ITG_TIMER_PRESCALER				TC_CLKSEL_DIV4_gc
	#elif ((SYS_CLK / ITG_UPDATE_FREQUENCY_HZ / 8UL) < 65535)
		#define		ITG_TIMER_PRESCALER_VALUE		8UL
		#define		ITG_TIMER_PRESCALER				TC_CLKSEL_DIV8_gc
	#elif ((SYS_CLK / ITG_UPDATE_FREQUENCY_HZ / 64UL) < 65535)
		#define		ITG_TIMER_PRESCALER_VALUE		64UL
		#define		ITG_TIMER_PRESCALER				TC_CLKSEL_DIV64_gc
	#elif ((SYS_CLK / ITG_UPDATE_FREQUENCY_HZ / 256UL) < 65535)
		#define		ITG_TIMER_PRESCALER_VALUE		256UL
		#define		ITG_TIMER_PRESCALER				TC_CLKSEL_DIV256_gc
	#else
		#error "No suitable prescaler for ITG update timer found"
	#endif
#endif


typedef enum
{
	ITG_ACTION_IDLE = 0,
	ITG_ACTION_FAILURE,
	ITG_ACTION_CHECK_COMM,
	ITG_ACTION_CONFIGURE,
	ITG_ACTION_GETTEMPXYZ,
	ITG_ACTION_NUMBER_OF_STATES
} E_ITG_ACTION_STATES;

#define		ITG_X_LEN				(2 - 1)
#define		ITG_TXYZ_LEN			(8 - 1)

// Register defines ******************************************************************************************************
#define		ITG						0xD2
#define 	ITG_WHO_AM_I			0x00
#define 	ITG_SMPLRT_DIV			0x15
#define 	ITG_DLPF_FS				0x16
#define 	ITG_INT_CFG				0x17
#define		ITG_INT_STATUS			0x1A
#define		ITG_TEMP_OUT_H			0x1B
#define		ITG_TEMP_OUT_L			0x1C
#define		ITG_GYRO_XOUT_H			0x1D
#define		ITG_GYRO_XOUT_L			0x1E
#define		ITG_GYRO_YOUT_H			0x1F
#define		ITG_GYRO_YOUT_L			0x20
#define		ITG_GYRO_ZOUT_H			0x21
#define		ITG_GYRO_ZOUT_L			0x22
#define		ITG_PWR_MGM				0x3E

// function prototypes
void						Init_ITG_TWI				();
void						Disable_ITG_TWI				();
void						Recover_ITG_TWI_from_ARBLost	();
uint8_t						Init_ITG_Sensor					();
void 						Get_ITGRawValues				(int16_t * DataBuffer);
int16_t						Get_ITG_Temperature				();
void						StartITGDataQuery				();
uint8_t						GetITGActionStatus				();
uint8_t						GetLastITGActionStatus 			();
uint8_t						StartITGAction 					(uint8_t ActionType, uint8_t	Block);
void						ITG_wait						(uint32_t ws);

void						ITG_Handle_TWI_ARB_Lost			();
void						ITG_Handle_TWI_BUS_Error		();
void						ITG_Handle_TWI_WIF				();
void						ITG_Handle_TWI_WIF_NACK			();
void						ITG_Handle_TWI_WIF_ACK			();
void						ITG_Handle_TWI_RIF				();
void						ITG_Handle_TWI_Unexpected		();

#endif /* ITG3200_H_ */


















