/*
 * lis03lv02dq.c
 *
 *  Created on: 08.10.2010
 *      Author: mkriegel
 */

#include "lis3lv02dq.h"

volatile uint8_t			Lis3LStatus = LIS_ACTION_IDLE;				// Current action status (see E_LIS_ACTION_STATES in header)
volatile uint8_t			LISByteCount = 0;							// internal byte counter
volatile uint8_t			LastLis3LActionStatus = LIS_ACTION_FAILURE;	// Result is 0 = Failure or successful action state defined in E_LIS_ACTION_STATES (see header)
volatile uint8_t		*	DataBuffer;									// Pointer to buffer where data shall be written to

volatile uint8_t			AccelByteData[6];							// Raw acceleration byte values 0,1=X - 2,3=Y - 4,5=Z
volatile int16_t			AccelRawData[3];							// Raw acceleration values 0=X 1=Y 2=Z
volatile int16_t			AccelCalibData[3];							// Calibrated acceleration values 0=X 1=Y 2=Z
volatile uint16_t			AccelVectorLength;							// Length of acceleration vector

/*****************************************************************************************************************************************/
ISR (LIS_UPDATE_TIMER_INT)
{
	if (Lis3LStatus == LIS_ACTION_IDLE)
		Start_LIS_Action(LIS_ACTION_GETXYZ, FUNC_NON_BLOCKING);
}

/*****************************************************************************************************************************************/
ISR (LIS_TWI_INT)
{
	LIS_Debug_Str	("LIS3int bytecount: ");											// Debug output
	LIS_Debug_u8	(LISByteCount);														// Debug output
	LIS_Debug_Str	(" ");																// Debug output

	uint8_t	status = LIS_TWI_PORT.MASTER.STATUS;										// Get status from TWI Master

	// Check bus state
	if (status & TWI_MASTER_ARBLOST_bm)													// Check if ARB was lost
		LIS_Handle_TWI_ARB_Lost();														// Recover from ARB lost

	else if (status & TWI_MASTER_BUSERR_bm)												// Check if BUS error occured
		LIS_Handle_TWI_BUS_Error();														// Recover from BUS error

	else if (status & TWI_MASTER_WIF_bm)												// Check if WIF (write interrupt flag) is set
		LIS_Handle_TWI_WIF();															// Handle TWI WIF

	else if (status & TWI_MASTER_RIF_bm)												// Check if RIF (read interrupt flag) is set
		LIS_Handle_TWI_RIF ();															// Handle TWI RIF

	else																				// Unexpected condition
		LIS_Handle_TWI_Unexpected();													// Handle it
}

/*************************************************************************************************************/
inline void LIS_Handle_TWI_Unexpected ()
{
	LIS_Debug_Str ("LIS unexpected condition\n");										// Debug output
	Lis3LStatus				= LIS_ACTION_FAILURE;										// Reset LIS status state machine
	LISByteCount				= 0;														// Reset ByteCount
	TWI_STOP (&LIS_TWI_PORT);															// Send TWI stop condition
}

/*************************************************************************************************************/
inline void	LIS_Handle_TWI_WIF_NACK ()
{
	LIS_ERROR_Debug_Str ("LIS NACK\n");													// Debug output
	TWI_STOP(&LIS_TWI_PORT);															// TWI stop condition
	LastLis3LActionStatus		= LIS_ACTION_FAILURE;									// Set last action result
	Lis3LStatus					= LIS_ACTION_IDLE;										// Reset LIS status state machine
}

/*************************************************************************************************************/
inline void LIS_Handle_TWI_ARB_Lost ()
{
	LIS_ERROR_Debug_Str ("LIS ARBlost\n");												// Debug output
	LIS_TWI_PORT.MASTER.STATUS |= TWI_MASTER_ARBLOST_bm;								// Reset ARB lost flag
	Lis3LStatus					= LIS_ACTION_IDLE;										// Reset LIS status state machine
	LISByteCount					= 0;													// Reset ByteCount

	LIS_Recover_TWI_from_ARBLost ();													// Recover from ARB lost
}

/*************************************************************************************************************/
inline void LIS_Handle_TWI_BUS_Error ()
{
	LIS_ERROR_Debug_Str ("LIS BUSerror\n");												// Debug output
	LIS_TWI_PORT.MASTER.STATUS |= TWI_MASTER_BUSERR_bm;									// Reset bus error flag
	Lis3LStatus					= LIS_ACTION_IDLE;										// Reset LIS status state machine
	LISByteCount					= 0;													// Reset ByteCount
}

/*************************************************************************************************************/
inline void LIS_Handle_TWI_WIF ()
{
	LIS_Debug_Str ("WIF ");																// Debug output

	if (LIS_TWI_PORT.MASTER.STATUS & TWI_MASTER_RXACK_bm)								// Check if NACK received
		LIS_Handle_TWI_WIF_NACK ();
	else																				// ACK received
		LIS_Handle_TWI_WIF_ACK();
}

/*************************************************************************************************************/
inline void LIS_Handle_TWI_WIF_ACK ()
{
	LIS_Debug_Str ("ACK\n");															// Debug output

	switch (Lis3LStatus)																// Switch "action" state machine status
	{
		// *****************************************************************************
		case LIS_ACTION_CHECK_COMM:														// COMM CHECK +++++++++++++++++++++++++++++++++++++++++++++
			TWI_STOP (&LIS_TWI_PORT);													// TWI stop condition
			LastLis3LActionStatus		= LIS_ACTION_CHECK_COMM;						// Set last action result
			Lis3LStatus					= LIS_ACTION_IDLE;								// Reset LIS status state machine
			LISByteCount					= 0;											// Reset bytecount
			LIS_Debug_Str ("LIS_ACTION_CHECK_COMM SUCCESSFUL - TWI stopped\n");			// Debug output
			break;
			// *****************************************************************************
		case LIS_ACTION_ENABLE:															// Enable LIS_ADDRESS sensor
			if (LISByteCount == 0)															// Send register address from which to read from
			{
				TWI_WRITE(&LIS_TWI_PORT, LIS_REG_CTRL_REG1);							// Write control register 1 address
				LISByteCount++;															// Increase ByteCount
			}
			else if (LISByteCount == 1)													// Write LIS_ADDRESS enable configuration bits
			{
				TWI_WRITE (&LIS_TWI_PORT, LIS_CONFIG1_BYTE);							// Enable power
				LISByteCount++;															// Increase ByteCount
			}
			else if (LISByteCount == 2)													// Configuration complete
			{
				TWI_STOP (&LIS_TWI_PORT);												// TWI stop condition
				LastLis3LActionStatus	= LIS_ACTION_ENABLE;							// Set last action result
				Lis3LStatus				= LIS_ACTION_IDLE;								// Reset LIS status state machine
				LISByteCount				= 0;											// Reset ByteCount
				LIS_Debug_Str ("LIS_ACTION_ENABLE SUCCESSFUL - TWI stopped\n");			// Debug output
			}
			break;
			// *****************************************************************************
		case LIS_ACTION_CONFIGURE:														// Configure LIS_ADDRESS sensor
			if (LISByteCount == 0)															// Send register address from which to read from
			{
				TWI_WRITE(&LIS_TWI_PORT, LIS_REG_CTRL_REG2);							// Write control register 1 address
				LISByteCount++;															// Increase ByteCount
			}
			else if (LISByteCount == 1)
			{
				TWI_WRITE (&LIS_TWI_PORT, LIS_CONFIG2_BYTE);							// Enable 2g scale, data ready generation
				LISByteCount++;															// Increase ByteCount
			}
			else if (LISByteCount == 2)
			{
				TWI_STOP (&LIS_TWI_PORT);												// TWI stop condition
				LastLis3LActionStatus		= LIS_ACTION_CONFIGURE;						// Set last action result
				Lis3LStatus					= LIS_ACTION_IDLE;							// Reset LIS status state machine
				LISByteCount					= 0;										// Reset ByteCount
				LIS_Debug_Str ("LIS_ACTION_CONFIGURE SUCCESSFUL - TWI stopped\n");		// Debug output
			}
			break;
			// *****************************************************************************
		case LIS_ACTION_GETXYZ:															// Get XYZ acceleration value
			if (LISByteCount == 0)															// Send register address from which to read from
			{
				TWI_WRITE(&LIS_TWI_PORT, LIS_REG_OUTX_L | LIS_FLAG_AUTOINC);			// Write register address of X low value and enable address auto increment
				LISByteCount++;															// Increase ByteCount
			}
			else if (LISByteCount == 1)													// Switch transmission to reading
			{
				//LIS_TWI_PORT.MASTER.CTRLC = TWI_MASTER_CMD_REPSTART_gc;				// Send repeated start condition (not needed as bus is already owned)
				TWI_START (&LIS_TWI_PORT, LIS_ADDRESS + I2C_READ);						// Send LIS_ADDRESS read address
				LISByteCount = 0;															// Reset ByteCount for reading
			}
			break;
	}
}

/*************************************************************************************************************/
inline void LIS_Handle_TWI_RIF ()
{
	LIS_Debug_Str ("RIF: ");																// Debug output
	uint8_t	data = LIS_TWI_PORT.MASTER.DATA;												// Read byte from TWI data register
	LIS_Debug_u8	(data);																	// Debug output
	LIS_Debug_Str	(" ");																	// Debug output

	switch (Lis3LStatus)																	// Switch "action" state machine status
	{
		case LIS_ACTION_GETXYZ:																// Read XYZ acceleration values
			if (LISByteCount < LIS_XYZ_BYTE_LEN)												// One of six 16er complement bytes
			{
				AccelByteData[LISByteCount] = data;											// Write received data byte to buffer
				TWI_ACK(&LIS_TWI_PORT);														// Issue ACK and continue reading
				LISByteCount++;																// Increment ByteCount
				LIS_Debug_Str ("\n");														// Debug output
			}
			else
			{
				AccelByteData[LISByteCount] = data;											// Write received data byte to buffer
				TWI_NACK_STOP(&LIS_TWI_PORT);												// Issue NACK and send stop condition
				LastLis3LActionStatus	= LIS_ACTION_GETXYZ;								// Set last action result
				Lis3LStatus				= LIS_ACTION_IDLE;									// Reset LIS status state machine
				LISByteCount				= 0;												// Reset ByteCount
				AccelRawData[0] 		= (AccelByteData[1] << 8) + AccelByteData[0];		// Calc X acceleration
				AccelRawData[1] 		= (AccelByteData[3] << 8) + AccelByteData[2];		// Calc Y acceleration
				AccelRawData[2] 		= (AccelByteData[5] << 8) + AccelByteData[4];		// Calc Z acceleration

				LIS_Debug_Str ("LIS_ACTION_GETXYZ OK\n");									// Debug output
			}
			break;
	}
}

/*************************************************************************************************************/
void LIS_Disable_TWI ()
{
	LIS_TWI_PORT.MASTER.BAUD = 0;
	LIS_TWI_PORT.MASTER.CTRLA = 0;
	LIS_TWI_PORT.MASTER.STATUS = TWI_MASTER_BUSSTATE_IDLE_gc;
}

/*************************************************************************************************************/
void LIS_Init_TWI ()
{
	LIS_TWI_PORT.MASTER.BAUD	= TWI_BAUD(SYS_CLK, LIS_TWI_FREQENCY_HZ);
	LIS_TWI_PORT.MASTER.CTRLA	= TWI_MASTER_INTLVL_LO_gc |
			TWI_MASTER_ENABLE_bm |
			TWI_MASTER_RIEN_bm |
			TWI_MASTER_WIEN_bm;
	LIS_TWI_PORT.MASTER.STATUS	= TWI_MASTER_BUSSTATE_IDLE_gc;
}

/*************************************************************************************************************/
void LIS_Recover_TWI_from_ARBLost()
{
	LIS_Disable_TWI ();																// Disable TWI

	LIS_PORT.DIRSET = (1 << PIN1);													// Set SCL pin as output
	LIS_PORT.DIRCLR = (1 << PIN0);													// Set SDA pin as input

	while (!(LIS_PORT.IN & (1 << PIN0)))											// Loop while SDA is low to recover from ARB lost
	{
		Debug_Str ("LIS RECOVER");
		LIS_PORT.OUTTGL = (1 << PIN1);												// Toggle SCL
		_delay_us(10);																// Wait 10µs
	}

	LIS_Init_TWI();																	// Initialize TWI
	LIS_ERROR_Debug_Str ("Recover_LIS_TWI_from_ARBLost\n");							// Output message

	Lis3LStatus				= LIS_ACTION_IDLE;										// Reset LIS status state machine
}

/*************************************************************************************************************/
uint8_t Init_LIS_Sensor ()
{
	LIS_Init_TWI ();																// Init LIS TWI

	Debug_Str ("Checking LIS communications:\n");									// Debug output
	Start_LIS_Action(LIS_ACTION_CHECK_COMM, FUNC_BLOCKING);						// Check if LIS is there, block this function
	if (Get_LIS_LastActionStatus() == LIS_ACTION_CHECK_COMM)						// LIS sensor found
		Debug_Str ("  - LIS3LV02DQ found ...\n");									// Debug output
	else																			// LIS sensor NOT found
	{
		Debug_u8(Get_LIS_LastActionStatus());										// Debug output
		Debug_Str ("  - FAILURE\n");												// Debug output
		return 0;																	// Quit program
	}

	_delay_ms (50);

	Debug_Str ("Configure LIS sensor:\n");
	Start_LIS_Action(LIS_ACTION_CONFIGURE, FUNC_BLOCKING);
	if (Get_LIS_LastActionStatus() == LIS_ACTION_CONFIGURE)
		Debug_Str ("  - LIS3LV02DQ configured ...\n");
	else
	{
		Debug_Str ("  - FAILURE\n");
		return 0;
	}

	_delay_ms (50);

	Debug_Str ("Enable LIS sensor:\n");
	Start_LIS_Action(LIS_ACTION_ENABLE, FUNC_BLOCKING);
	if (Get_LIS_LastActionStatus() == LIS_ACTION_ENABLE)
		Debug_Str ("  - LIS3LV02DQ enabled ...\n\n");
	else
	{
		Debug_Str ("  - FAILURE\n");
		return 0;
	}

	_delay_ms (50);

	return 1;
}

/*************************************************************************************************************/
void Get_LISRawValues (int16_t * DataBuffer)
{
	uint8_t i;

	cli ();

	for (i = 0; i < 3; i++)
		DataBuffer[i] = AccelRawData[i];

	sei ();

	LastLis3LActionStatus = 0;
}

/*************************************************************************************************************/
inline void Start_LIS_Data_Query ()
{
	// Init LIS_ADDRESS timer
	TC_SetPeriod 		(&LIS_UPDATE_TIMER, SYS_CLK / LIS_UPDATE_FREQUENCY_HZ / LIS_TIMER_PRESCALER_VALUE);
	TC_SetPrescaler		(&LIS_UPDATE_TIMER, LIS_TIMER_PRESCALER);
	TC_SetOverflowInt	(&LIS_UPDATE_TIMER, TC_OVFINTLVL_HI_gc);
}

/*************************************************************************************************************/
inline uint8_t Get_LIS_LastActionStatus ()
{
	return LastLis3LActionStatus;
}

/*************************************************************************************************************/
inline uint8_t	Get_LIS_ActionStatus ()
{
	return Lis3LStatus;
}

/*************************************************************************************************************/
uint8_t	Start_LIS_Action (uint8_t ActionType, uint8_t Blocking)
{
	LIS_Debug_Str	("LISact: ");
	LIS_Debug_u8	(ActionType);
	LIS_Debug_Str	("\n");

	if (ActionType > LIS_ACTION_NUMBER_OF_STATES)
	{
		LIS_Debug_Str ("LIS3L action is unknown\n");
		Lis3LStatus = LIS_ACTION_IDLE;
		return Lis3LStatus;
	}
	else if (Lis3LStatus != LIS_ACTION_IDLE)
	{
		LIS_Debug_Str ("-");
		return Lis3LStatus;
	}
	else
	{
		Lis3LStatus		= ActionType;
		LISByteCount		= 0;
	}

	LIS_Debug_Str ("TWI bus status: ");
	switch (LIS_TWI_PORT.MASTER.STATUS & TWI_MASTER_BUSSTATE_gm)
	{
		case TWI_MASTER_BUSSTATE_UNKNOWN_gc:
			LIS_Debug_Str ("LIS BUS STATE UNKNOWN\n");
			break;

		case TWI_MASTER_BUSSTATE_IDLE_gc:
			LIS_Debug_Str ("LIS BUS STATE IDLE\n");
			break;

		case TWI_MASTER_BUSSTATE_OWNER_gc:
			LIS_Debug_Str ("LIS BUS STATE OWNER\n");
			break;

		case TWI_MASTER_BUSSTATE_BUSY_gc:
			LIS_Debug_Str ("LIS BUS STATE BUSY\n");
			break;
	}

	// Start condition + LIS_ADDRESS address
	TWI_START (&LIS_TWI_PORT, LIS_ADDRESS + I2C_WRITE);

	// Loop until done if blocking flag is set
	if (Blocking)
	{
		while (Lis3LStatus != LIS_ACTION_IDLE)
		{
			Debug_Str ("BLOCK");
			asm volatile ("nop");
		}
	}

	return Lis3LStatus;
}














