#ifndef _LIS3LV02DQ_H__
#define _LIS3LV02DQ_H__

#include <avr/io.h>
#include <avr/interrupt.h>
#include <math.h>
#include <util/delay.h>
#include "../tricopter_config.h"
#include "../m_twi.h"
#include "../m_timer.h"
#include "../debug.h"

#define		LIS_1G_SCALE						1024UL

#if (LIS_DEBUGGING == 1)
#define		LIS_Debug_Str(_str)					( Debug_Str(_str) )
#define		LIS_Debug_u8(_u8)					( Debug_u8(_u8) )
#else
#define		LIS_Debug_Str(_str)
#define		LIS_Debug_u8(_u8)
#endif

#if (LIS_ERROR_DEBUGGING == 1)
#define 	LIS_ERROR_Debug_Str(_str)			( Debug_Str(_str) )
#else
#define		LIS_ERROR_Debug_Str(_str)
#endif

// Timer settings calculation -- do NOT change
#ifndef SYS_CLK
	#error "SYS_CLK: System clock frequency not defined!!!"
#endif

#if ((SYS_CLK / LIS_UPDATE_FREQUENCY_HZ) < 65535)
	#define		LIS_TIMER_PRESCALER_VALUE		1UL
	#define		LIS_TIMER_PRESCALER			TC_CLKSEL_DIV1_gc
#elif ((SYS_CLK / LIS_UPDATE_FREQUENCY_HZ / 2UL) < 65535)
	#define		LIS_TIMER_PRESCALER_VALUE		2UL
	#define		LIS_TIMER_PRESCALER			TC_CLKSEL_DIV2_gc
#elif ((SYS_CLK / LIS_UPDATE_FREQUENCY_HZ / 4UL) < 65535)
	#define		LIS_TIMER_PRESCALER_VALUE		4UL
	#define		LIS_TIMER_PRESCALER			TC_CLKSEL_DIV4_gc
#elif ((SYS_CLK / LIS_UPDATE_FREQUENCY_HZ / 8UL) < 65535)
	#define		LIS_TIMER_PRESCALER_VALUE		8UL
	#define		LIS_TIMER_PRESCALER			TC_CLKSEL_DIV8_gc
#elif ((SYS_CLK / LIS_UPDATE_FREQUENCY_HZ / 64UL) < 65535)
	#define		LIS_TIMER_PRESCALER_VALUE		64UL
	#define		LIS_TIMER_PRESCALER			TC_CLKSEL_DIV64_gc
#elif ((SYS_CLK / LIS_UPDATE_FREQUENCY_HZ / 256UL) < 65535)
	#define		LIS_TIMER_PRESCALER_VALUE		256UL
	#define		LIS_TIMER_PRESCALER			TC_CLKSEL_DIV256_gc
#else
	#error "No suitable prescaler for LIS update timer found"
#endif


typedef enum
{
	LIS_ACTION_IDLE = 0,
	LIS_ACTION_FAILURE,
	LIS_ACTION_CHECK_COMM,
	LIS_ACTION_ENABLE,
	LIS_ACTION_CONFIGURE,
	LIS_ACTION_GETXYZ,
	LIS_ACTION_NUMBER_OF_STATES
} E_LIS_ACTION_STATES;

#define		LIS_X_BYTE_LEN			(2 - 1)
#define		LIS_XYZ_BYTE_LEN		(6 - 1)

// Register defines ******************************************************************************************************
#define		LIS_ADDRESS				0x3A
#define		LIS_FLAG_AUTOINC		0x80
#define 	LIS_REG_WHO_AM_I		0x0F
#define 	LIS_REG_CTRL_REG1		0x20
#define 	LIS_REG_CTRL_REG2		0x21
#define 	LIS_REG_CTRL_REG3		0x22
#define		LIS_REG_OUTX_L			0x28
#define		LIS_REG_OUTX_H			0x29
#define		LIS_REG_OUTY_L			0x2A
#define		LIS_REG_OUTY_H			0x2B
#define		LIS_REG_OUTZ_L			0x2C
#define		LIS_REG_OUTZ_H			0x2D

// CTRLREG1 bit masks ******************************************************************************************************
#define		XEN						(1 << 0)
#define		YEN						(1 << 1)
#define		ZEN						(1 << 2)
#define		SELFTEST				(1 << 3)
#define		RATE_40HZ				((0 << 4) | (0 << 5))
#define		RATE_160HZ				((1 << 4) | (0 << 5))
#define		RATE_640HZ				((0 << 4) | (1 << 5))
#define		RATE_2560HZ				((1 << 4) | (1 << 5))
#define		DEVICE_ON				((1 << 6) | (1 << 7))

#define		LIS_CONFIG1_BYTE		( XEN | YEN | ZEN | RATE_160HZ | DEVICE_ON )

// CTRLREG2 bit masks ******************************************************************************************************
#define		SCALE_2G				(0 << 7)
#define		SCALE_6G				(1 << 7)
#define		BDU						(1 << 6)
#define		BLE						(1 << 5)
#define		BOOT					(1 << 4)
#define		IEN						(1 << 3)
#define		DRDY					(1 << 2)
#define		DAS						(1 << 0)

#define		LIS_CONFIG2_BYTE		( SCALE_2G | DRDY )

/*******************************************************************************************************************************************************/
/*******************************************************************************************************************************************************/
/*******************************************************************************************************************************************************/
/*******************************************************************************************************************************************************/

// function prototypes
uint8_t				Init_LIS_Sensor					();
uint8_t				Get_LIS_ActionStatus			();
uint8_t				Get_LIS_LastActionStatus 		();
void				Start_LIS_Data_Query			();
uint8_t				Start_LIS_Action 				(uint8_t ActionType, uint8_t	Block);
void				Get_LISRawValues				(int16_t * DataBuffer);

// Basic TWI functions
void				LIS_Init_TWI					();
void				LIS_Disable_TWI					();

// Interrupt and error handling TWI functions
void				LIS_Handle_TWI_ARB_Lost			();
void				LIS_Recover_TWI_from_ARBLost	();
void				LIS_Handle_TWI_BUS_Error		();
void				LIS_Handle_TWI_Unexpected		();
void				LIS_Handle_TWI_WIF 				();
void				LIS_Handle_TWI_RIF				();
void				LIS_Handle_TWI_WIF_NACK			();
void				LIS_Handle_TWI_WIF_ACK			();
void 				LIS_Handle_TWI_RIF				();

// debug function prototypes
void				debug							(char ch);
void				debugu8 						(uint8_t ch);
void 				debugStr						(char * str);
void 				ErrorDebugStr					(char * str);

#endif













